@@ -401,6 +401,32 @@ tf_channel_dispose (GObject *object)
 
   g_debug (G_STRFUNC);
 
+  if (self->priv->streams)
+    {
+      guint i;
+
+      for (i = 0; i < self->priv->streams->len; i++)
+        {
+          GObject *obj = g_ptr_array_index (self->priv->streams, i);
+
+          if (obj != NULL)
+            {
+              tf_stream_error (TF_STREAM (obj),
+                  TP_MEDIA_STREAM_ERROR_UNKNOWN,
+                  "UI stopped channel");
+
+              /* this first one covers both error and closed */
+              g_signal_handlers_disconnect_by_func (obj,
+                  stream_closed_cb, self);
+
+              g_object_unref (obj);
+            }
+        }
+
+      g_ptr_array_free (self->priv->streams, TRUE);
+      self->priv->streams = NULL;
+    }
+
   if (self->priv->sessions != NULL)
     {
       guint i;
@@ -430,28 +456,6 @@ tf_channel_dispose (GObject *object)
       self->priv->sessions = NULL;
     }
 
-  if (self->priv->streams)
-    {
-      guint i;
-
-      for (i = 0; i < self->priv->streams->len; i++)
-        {
-          GObject *obj = g_ptr_array_index (self->priv->streams, i);
-
-          if (obj != NULL)
-            {
-              /* this first one covers both error and closed */
-              g_signal_handlers_disconnect_by_func (obj,
-                  stream_closed_cb, self);
-
-              g_object_unref (obj);
-            }
-        }
-
-      g_ptr_array_free (self->priv->streams, TRUE);
-      self->priv->streams = NULL;
-    }
-
   if (self->priv->channel_ready_idle)
     {
       g_source_remove (self->priv->channel_ready_idle);
@@ -682,7 +686,7 @@ new_stream_cb (TfSession *session,
       gchar *str = g_strdup_printf ("failed to construct TpMediaStreamHandler:"
           " bad object path '%s'?", object_path);
       g_warning ("%s", str);
-      tf_channel_error (self, 0, str);
+      tf_channel_error (self, TP_MEDIA_STREAM_ERROR_UNKNOWN, str);
       g_free (str);
       return;
     }
@@ -705,12 +709,6 @@ new_stream_cb (TfSession *session,
 
   fs_codec_list_destroy (local_codec_config);
 
-  if (!stream)
-    {
-      tf_channel_error (self, 0, "Error creating stream");
-      return;
-    }
-
   g_object_unref (proxy);
   g_object_unref (fs_conference);
   g_object_unref (fs_participant);
@@ -723,7 +721,7 @@ new_stream_cb (TfSession *session,
       g_warning ("connection manager gave us a new stream with existing id "
           "%u, sending error!", stream_id);
 
-      tf_stream_error (stream, 0,
+      tf_stream_error (stream, TP_MEDIA_STREAM_ERROR_INVALID_CM_BEHAVIOR,
           "already have a stream with this ID");
 
       g_object_unref (stream);
@@ -740,21 +738,9 @@ static void
 session_invalidated_cb (TfSession *session, gpointer user_data)
 {
   TfChannel *self = TF_CHANNEL (user_data);
-  FsConference *conf = NULL;
-  FsParticipant *part = NULL;
 
   g_signal_handlers_disconnect_by_func (session, new_stream_cb, self);
 
-  g_object_get (session,
-      "farsight-conference", &conf,
-      "farsight-participant", &part,
-      NULL);
-
-  g_signal_emit (self, signals[SESSION_INVALIDATED], 0, conf, part);
-
-  g_object_unref (conf);
-  g_object_unref (part);
-
   g_object_unref (session);
   g_ptr_array_remove_fast (self->priv->sessions, session);
 }
@@ -780,13 +766,10 @@ add_session (TfChannel *self,
 
   if (proxy == NULL)
     {
-      gchar *str = g_strdup_printf ("failed to construct TpMediaSessionHandler:"
-          " %s",
-          error->message);
+      g_prefix_error (&error,"failed to construct TpMediaSessionHandler: ");
+      g_warning ("%s", error->message);
+      tf_channel_error (self, TP_MEDIA_STREAM_ERROR_UNKNOWN, error->message);
       g_error_free (error);
-      g_warning ("%s", str);
-      tf_channel_error (self, 0, str);
-      g_free (str);
       return;
     }
 
@@ -794,12 +777,10 @@ add_session (TfChannel *self,
 
   if (session == NULL)
     {
-      gchar *str = g_strdup_printf("failed to create session: %s",
-          error->message);
+      g_prefix_error (&error, "failed to create session: ");
+      g_warning ("%s", error->message);
+      tf_channel_error (self, fserror_to_tperror (error), error->message);
       g_error_free (error);
-      g_warning ("%s", str);
-      tf_channel_error (self, 0, str);
-      g_free (str);
       return;
     }
 
