@@ -133,7 +133,8 @@ enum
   PROP_NAT_PROPERTIES,
   PROP_SINK_PAD,
   PROP_LOCAL_PREFERENCES,
-  PROP_TOS
+  PROP_TOS,
+  PROP_RESOURCES
 };
 
 static void get_all_properties_cb (TpProxy *proxy,
@@ -266,6 +267,9 @@ tf_stream_get_property (GObject    *object,
       else
         g_value_set_uint (value, self->priv->tos);
       break;
+    case PROP_RESOURCES:
+      g_value_set_uint (value, self->priv->has_resource);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -540,6 +544,15 @@ tf_stream_class_init (TfStreamClass *klass)
           0, 255, 0,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class, PROP_RESOURCES,
+      g_param_spec_uint ("resources",
+          "Resources held by the stream",
+          "The resources held by a TpMediaStreamDirection",
+          TP_MEDIA_STREAM_DIRECTION_NONE,
+          TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL,
+          TP_MEDIA_STREAM_DIRECTION_NONE,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * TfStream::closed:
    * @stream: the stream that has been closed
@@ -1257,6 +1270,13 @@ fs_codecs_to_tp (TfStream *stream,
                                g_strdup (param->value));
         }
 
+      if (fsc->ABI.ABI.ptime)
+        g_hash_table_insert (params, g_strdup ("ptime"),
+            g_strdup_printf ("%u", fsc->ABI.ABI.ptime));
+      if (fsc->ABI.ABI.maxptime)
+        g_hash_table_insert (params, g_strdup ("maxptime"),
+            g_strdup_printf ("%u", fsc->ABI.ABI.maxptime));
+
       g_value_init (&codec, TP_STRUCT_TYPE_MEDIA_STREAM_HANDLER_CODEC);
       g_value_take_boxed (&codec,
           dbus_g_type_specialized_construct (TP_STRUCT_TYPE_MEDIA_STREAM_HANDLER_CODEC));
@@ -1386,7 +1406,12 @@ fill_fs_params (gpointer key, gpointer value, gpointer user_data)
 {
   FsCodec *codec = user_data;
 
-  fs_codec_add_optional_parameter (codec, key, value);
+  if (!strcmp (key, "ptime") && atoi (value) > 0)
+    codec->ABI.ABI.ptime = atoi (value);
+  else if  (!strcmp (key, "maxptime") && atoi (value) > 0)
+    codec->ABI.ABI.maxptime = atoi (value);
+  else
+    fs_codec_add_optional_parameter (codec, key, value);
 }
 
 static void
